/*
	ISCatalogInterface JavaScript API
	Copyright (c) 2012-2014 IOSPIRIT GmbH. All rights reserved.
*/

/* Catalog API class*/
function Catalog()
{
}

Catalog.prototype.setup = function()
{
	this.name = "Shared Catalog";
	this.reloadAvailableItems();
	this.activeSection = window.catalogAPI.manage("activeSection");
	window.catalogAPI.enableDownloadProgressUpdates(true);
	this._didAutoPickFirstUnread = false;
	this.render();
}

// Available Items
Catalog.prototype.availableItems;
Catalog.prototype.reloadAvailableItems = function()
{
	var itemDescriptions = this.decodeAPIResult(window.catalogAPI.availableItems(1));
	var newItems = [];
	
	for (var i=0; i<itemDescriptions.length; i++)
	{
		var itemObj;
	
		itemObj = new CatalogItem(itemDescriptions[i]);	
		
		newItems.push(itemObj);
	}

	this.availableItems = newItems;
}

Catalog.prototype.itemWithIdentifier = function(itemID)
{
	for (var i=0; i<this.availableItems.length; i++)
	{
		var itemObj = this.availableItems[i];
		
		if (itemObj.identifier == itemID)
		{
			return (itemObj);
		}
	}
	
	return (null);
}

// Localization
Catalog.prototype.localize = function(localizeString)
{
	return(window.catalogAPI.localize(localizeString));
}

// Tools
Catalog.prototype.name;
Catalog.prototype.decodeAPIResult = function(apiResult)
{
	// alert ("Decoding JSON: " + apiResult);
	return (eval('(' + apiResult + ')'));
}

// Render
Catalog.prototype.renderContents = function(itemObj,fieldName)
{
	switch (fieldName)
	{
		case "title":
			var additional = "";
		
			if (itemObj.installerRequiresRelaunch)
			{
				if (additional!="") { additional += ", "; }
				additional += this.localize("Relaunch required");
			}

			if (itemObj.installerRequiresReboot)
			{
				if (additional!="") { additional += ", "; }
				additional += this.localize("Reboot required");
			}
					
			return ('<span class="name">' + itemObj.name + '</span> <span class="version">' + itemObj.version + '</span>' + 
				((additional!="") ? '<span class="more"> &mdash; ' + additional + "</span>" : ""));
		break;

		case "status":
			if (itemObj.installed == 1)
			{
				return (this.localize("Installed"));
			}
			else
			{
				return (((itemObj.sizeLocalized=="") ? this.localize("Unknown size") : itemObj.sizeLocalized) +
					(((itemObj.localCopyDownloaded===true) || (itemObj.localCopyComplete===true)) ? " &mdash; " + this.localize("Downloaded") : ""));
			}
		break;
		
		case "action":
			if (itemObj.installed == 1)
			{
				return ('<div><a href="#" onclick="catalog.runAction(catalog.getItemIDFromParent(this),\'launch\'); return(false);">' + this.localize("Launch") + '</a></div>');
			}
			else
			{
				switch (itemObj.status)
				{
					case "installing":
					case "pendingInstallation":
					case "unavailable":
						var caption = "";
					
						switch (itemObj.status)
						{
							case "installing":
								caption = this.localize("Installing..");
							break;

							case "pendingInstallation":
								caption = this.localize("Waiting..");
							break;

							case "unavailable":
								caption = this.localize("Install");
							break;
						}

						return ('<div disabled="true"><a>' + caption + '</a></div>');
					break;

					case "installed":
					break;

					case "available":
					case "downloaded":
					case "downloading":
					default:
						return ('<div><a href="#" onclick="catalog.runAction(catalog.getItemIDFromParent(this),\'install\'); return(false);">' + this.localize("Install") + '</a>' +
							
							((itemObj.category == "updates") ? 
								'<select onmousedown="this.selectedIndex = -1;" onclick="catalog.handleActionDropDown(this); return(false);">' +
								'	<option action="ignore">' + this.localize("Hide") + '</option>' +
								'</select>'
							: '') + 
							
							'</div>');
					break;
				}
			}
		break;
		
		case "description":
			var content = "";
			
			if ((itemObj.type == "message") || (itemObj.category == "more"))
			{
				if (itemObj.descriptionURL)
				{
					return('<iframe src="' + itemObj.descriptionURL + '" frameborder="0" width="100%" height="100%">' + itemObj.description + '</iframe>');
				}
				else
				{
					content = itemObj.description;
				}
			}
			else
			{
				if (itemObj.description)
				{
					content = itemObj.description;
				}
				else
				{
					content = '<div class="loading">' + this.localize("Loading..") + '</div>';
				}
			}

			return ('<div class="content">' + content + '</div>');
		break;
	}
	
	return ("");
}

Catalog.prototype.sourcelist;
Catalog.prototype._fullscreenObj;
Catalog.prototype.availableItemsForSection = function(sectionID)
{
	var returnItemObjs = [];

	if (this.availableItems.length > 0)
	{
		for (var i=0; i<this.availableItems.length; i++)
		{
			var itemObj = this.availableItems[i];
			
			switch (sectionID)
			{
				case "updates":
					if ((itemObj.type == "item") && (itemObj.category == "updates"))
					{
						returnItemObjs.push(itemObj);
					}
				break;

				case "news":
					if (itemObj.type == "message")
					{
						returnItemObjs.push(itemObj);
					}
				break;

				case "more":
					if ((itemObj.type == "item") && (itemObj.category == "more"))
					{
						returnItemObjs.push(itemObj);
					}
				break;
			}
		}
	}
	
	return (returnItemObjs);
}

Catalog.prototype._didAutoPickFirstUnread;
Catalog.prototype.render = function()
{
	var sectionObj, rowObj;
	var myself = this;
	var sectionItems = this.availableItemsForSection(this.activeSection);
	
	if (this._fullscreenObj == undefined)
	{
		var fullscreenObj;
		
		if ((fullscreenObj = document.getElementById("catalogfullscreen")) !== null)
		{
			this._fullscreenObj = fullscreenObj;
		}
	}
	
	if (this.sourcelist === undefined)
	{
		var sourcelistObj;
		
		if ((sourcelistObj = document.getElementById("catalogsourcelist")) !== null)
		{
			this.sourcelist = new ISSourceList(sourcelistObj);
			this.sourcelist.hideSectionHeaders = true;
		}
	}
	else
	{
		this.sourcelist.clear();
	}
	
	if (sectionItems.length > 0)
	{
		var sectionObj;

		sectionObj = this.sourcelist.addSection(this.activeSection, "");
		sectionObj.allowEmptySelection = true;
	
		for (var i=0; i<sectionItems.length; i++)
		{
			var itemObj = sectionItems[i];
			var rowObj;
			
			switch (this.activeSection)
			{
				case "updates":
				case "more":
					rowObj = sectionObj.addRow(itemObj.identifier, itemObj.name + ' <span>' + itemObj.version + '</span>');
					rowObj.imageURL = itemObj.imageURL;
				break;

				case "news":
					rowObj = sectionObj.addRow(itemObj.identifier, itemObj.name);
					rowObj.imageURL = ((itemObj.messageFlagsRead == 1) ? "iconRead.png" : "iconUnread.png");
					
					if (this._didAutoPickFirstUnread === false)
					{
						if (sectionObj.selectedRowID === null)
						{
							if (itemObj.messageFlagsRead == 0)
							{
								this._didAutoPickFirstUnread = true;
								sectionObj.selectedRowID = itemObj.identifier;
							}
						}
					}
				break;
			}
		}

		sectionObj.allowEmptySelection = false;
		
		if (sectionItems.length == 1)
		{
			this.showSideBar(false);
		}
		else
		{
			this.showSideBar(true);
		}
		
		this._fullscreenObj.style.display = "none";
	}
	else
	{
		var message = "";
		var noNews = "", addButton = "", reloadButtonText = "Reload";
	
		switch (window.catalogAPI.currentRepositoryState())
		{
			case "initialized":
			case "updating":
				message = this.localize("Checking for updates..");
			break;
			
			case "updateFailed":
				noNews = this.localize("Checking for updates failed.");
				reloadButtonText = "Retry";
			
			case "ready":
				if (noNews == "")
				{
					switch (this.activeSection)
					{
						case "updates":
							if (this.ignoreListCount() > 0)
							{
								noNews = this.localize("No new updates available.") + " " + this.ignoreListCount() + " " + this.localize("item(s) hidden.");
								addButton = '<button onclick="catalog.resetIgnoreList();">' + this.localize("Show all items") + '</button>&nbsp;'
							}
							else
							{
								noNews = this.localize("You're up to date!");
							}
						break;
	
						case "news":
							noNews = this.localize("No news.");
						break;
	
						case "more":
							noNews = this.localize("No new downloads available.");
						break;
					}
				}
	
				message = noNews + '<br><br>' + addButton + '<button onclick="catalog.checkForUpdates();">' + this.localize(reloadButtonText) + '</button>';
			break;
		}

		this._fullscreenObj.style.display = "table";
		this._fullscreenObj.innerHTML = "<div>" + message + "</div>";
	}
	
	this.sourcelist.onSelectionChange = function(sectionID, rowID)
	{
		myself.present(sectionID, rowID);
	};
	
	this.updateCounts();
	this.sourcelist.render();
}

Catalog.prototype.updateCounts = function()
{
	if (this.sourcelist !== undefined)
	{
		var sectionObj;
	
		if (this.availableItems.length > 0)
		{
			var updatesCnt = 0, newsCnt = 0, moreCnt = 0;
	
			for (var i=0; i<this.availableItems.length; i++)
			{
				var itemObj = this.availableItems[i];
				var sectionObj = null, rowObj;
				
				switch (itemObj.type)
				{
					case "item":
						switch (itemObj.category)
						{
							case "updates":
								updatesCnt++;
							break;
	
							case "more":
								moreCnt++;
							break;
						}
					break;
					
					case "message":
						if (itemObj.messageFlagsRead == 0)
						{
							newsCnt++;
						}
					break;
				}
			}
		}
		
		if ((sectionObj = this.sourcelist.getSectionByID("updates")) != null)
		{
			sectionObj.badge = (updatesCnt > 0) ?  updatesCnt : null;
		}

		if ((sectionObj = this.sourcelist.getSectionByID("news")) != null)
		{
			sectionObj.badge = (newsCnt > 0) ?  newsCnt : null;
		}
	}
}

Catalog.prototype.present = function(sectionID, rowID)
{
	var contentHTML = "";

	switch (sectionID)
	{
		case "updates":
		case "more":
			{
				var itemObj;
				
				if ((itemObj = this.itemWithIdentifier(rowID)) !== null)
				{
					var itemHTML = "";
			
					itemHTML += '<div class="item" itemID="' + itemObj.identifier + '" id="' + itemObj.identifier.toLowerCase() + '">';
					itemHTML += '	<div class="info">';
					itemHTML += '		<div class="image" style="background-image: url(\'' + itemObj.imageURL + '\')"></div>';
					itemHTML += '		<div class="title">' + this.renderContents(itemObj, "title") + '</div>';
					itemHTML += '		<div class="status">' + this.renderContents(itemObj, "status") + '</div>';
					itemHTML += '		<div class="release">' + this.localize("Released on") + ' ' + itemObj.date + '</div>';
					itemHTML += '		<div class="action">' + this.renderContents(itemObj, "action") + '</div>';
					itemHTML += '	</div>';
				
					itemHTML += '	<div class="description">';
					itemHTML += this.renderContents(itemObj, "description");
					if (itemObj.description === null)
					{
						this.requestContent(itemObj.identifier, "description");
					}
					itemHTML += '	</div>';

					itemHTML += '</div>';
					
					contentHTML += itemHTML;
				}
			}
		break;
		
		case "news":
			{
				var itemObj;
				
				if ((itemObj = this.itemWithIdentifier(rowID)) !== null)
				{
					var itemHTML = "";
			
					itemHTML += '<div class="item" itemID="' + itemObj.identifier + '" id="' + itemObj.identifier.toLowerCase() + '">';
				
					itemHTML += '	<div class="description noinfo">';
					itemHTML += this.renderContents(itemObj, "description");
					// For news, we now use the URL in an iFrame
					/*
					if (itemObj.description === null)
					{
						this.requestContent(itemObj.identifier, "description");
					}
					*/
					itemHTML += '	</div>';
					itemHTML += '</div>';
					
					contentHTML += itemHTML;
					
					itemObj.messageFlagsRead = 1;
					this.updateCounts();
				}
			}
		break;
	}
	
	document.getElementById("catalogdetail").innerHTML = contentHTML;
}

Catalog.prototype.activeSection;
Catalog.prototype.switchToSection = function(sectionID)
{
	if (this.activeSection != sectionID)
	{
		this.activeSection = sectionID;
		this.render();
	}
}

Catalog.prototype.showSideBar = function(showSideBar)
{
	var masterObj, detailObj;
	
	if ((masterObj = document.getElementById("catalogsourcelist")) !== null)
	{
		if ((detailObj = document.getElementById("catalogdetail")) !== null)
		{
			if (showSideBar)
			{
				masterObj.style.left = "0px";
				detailObj.style.left = parseInt(masterObj.getAttribute("sidebarwidth")) + "px";
				detailObj.style.borderLeftWidth = "1px";
			}
			else
			{
				masterObj.style.left = "-" + parseInt(masterObj.getAttribute("sidebarwidth")) + "px";
				detailObj.style.left = "0px";
				detailObj.style.borderLeftWidth = "0px";
			}
		}
	}
}

Catalog.prototype.refresh = function()
{
	this.reloadAvailableItems();
	this.render();
}

Catalog.prototype.updateContents = function(itemID, fieldName, newContent)
{
	var itemDOMObj;

	if ((itemDOMObj = document.getElementById(itemID.toLowerCase())) !== null)
	{
		var elementObj = itemDOMObj.getElementsByClassName(fieldName)[0];
		
		if (elementObj !== undefined)
		{
			itemDOMObj.getElementsByClassName(fieldName)[0].innerHTML = newContent;
		}
	}
}

Catalog.prototype.requestContent = function(itemID, fieldName)
{
	return (window.catalogAPI.requestContentForItemField(itemID, fieldName));
}

Catalog.prototype.reportProgressUpdate = function(progressInfo)
{
	var newContent = "";

	if (progressInfo['error'] !== undefined)
	{
		newContent = progressInfo['error'];
	}
	else
	{
		if (progressInfo['progressIndeterminate'] !== false)
		{
			newContent = this.localize("Working..");
		}
		else
		{
			newContent = '<div class="progressbar"><span style="width:' + Math.ceil(progressInfo['progress']*100) + '\%;"></span></div> ' + progressInfo['progressMessage'];
		}
		
		if (progressInfo['progress']==1)
		{
			this.reloadAvailableItems();

			newContent = this.renderContents(this.itemWithIdentifier(progressInfo['itemID']),"status");

			this.updateContents(progressInfo['itemID'],"action", this.renderContents(this.itemWithIdentifier(progressInfo['itemID']),"action"));
		}
	}

	this.updateContents(progressInfo['itemID'], "status", newContent);
}

Catalog.prototype.reportItemUpdate = function(statusInfo)
{
	var itemObj = this.itemWithIdentifier(statusInfo['itemID']);

	if (itemObj !== null)
	{
		switch (statusInfo['fieldName'])
		{
			case "status":
				itemObj.status = statusInfo['status'];
				this.updateContents(itemObj.identifier, "action", this.renderContents(itemObj, "action"));
			break;
			
			case "description":
				itemObj.description = statusInfo['description'];
				
				if (statusInfo['error'] !== undefined)
				{
					itemObj.description = '<span title="' + (statusInfo['error'].replace(/(["'])/g, "'")) + '">' + this.localize("Error loading content") + '</span>. <a href="#" onclick="catalog.requestContent(\'' + itemObj.identifier + '\', \'description\'); return(false);">' + this.localize("Retry") + '</a>';
				}
				
				this.updateContents(itemObj.identifier, "description", this.renderContents(itemObj, "description"));
			break;
		}
	}
}

Catalog.prototype.reportTaskUpdate = function(task, event)
{
	// alert("TASK UPDATE" + task + " " + event);
}

Catalog.prototype.reportRepositoryStateUpdate = function(newState)
{
	/*
	if (this.availableItems.length == 0)
	{
		switch (newState)
		{
			case "updating":
			case "updateFailed":
			case "ready":
				this.render();
			break;
		}
	}
	*/
}

/* Item Actions */
Catalog.prototype.runAction = function(itemID, action)
{
	// Handle case where itemID is null
	if (itemID === null)
	{
		return;
	}
	
	window.catalogAPI.runAction(action, itemID);

	// alert (action + " " + itemID);
}

Catalog.prototype.handleActionDropDown = function(element)
{
	var action = null;

	if (element.selectedIndex != -1)
	{
		if ((action = element.options[element.selectedIndex].getAttribute("action")) !== null)
		{
			this.runAction(this.getItemIDFromParent(element), action);
		}
	}
}

Catalog.prototype.getItemIDFromParent = function(element)
{
	while ((element.getAttribute("itemID") === null) && (element !== undefined))
	{
		element = element.parentNode;
	}
	
	if (element !== undefined)
	{
		return (element.getAttribute("itemID"));
	}
	
	return (null);
}

/* Management */
Catalog.prototype.checkForUpdates = function()
{
	this.manage("checkForUpdates");
}

Catalog.prototype.ignoreListCount = function()
{
	return (this.manage("ignoreListCount"));
}

Catalog.prototype.resetIgnoreList = function()
{
	return (this.manage("ignoreListReset"));
}

Catalog.prototype.manage = function(manageCommand)
{
	return (window.catalogAPI.manage(manageCommand));
}

/* Globally shared catalog object */
var catalog;

function InitCatalog()
{
	catalog = new Catalog();
	catalog.setup();
}

addEventListener("DOMContentLoaded", function(event) { InitCatalog(); }, false); // DOMContentLoaded fires even before onLoad

/* Catalog Item */
function CatalogItem(itemDesc)
{
	if (itemDesc)
	{
		this._updateWithItemDesc(itemDesc);
	}
}

CatalogItem.prototype = {
   	get messageFlagsRead()
   	{
   		return (this._messageFlagsRead);
    	},
    		
    	set messageFlagsRead(newMessageFlagsRead)
    	{
    		if (newMessageFlagsRead != this._messageFlagsRead)
    		{
	   		this._messageFlagsRead = newMessageFlagsRead;
	   		
	   		if (catalog !== undefined)
	   		{
	   			catalog.runAction(this.identifier, ((newMessageFlagsRead==0) ? "markMessageUnread" : "markMessageRead"));
		   	}
			else
			{
				alert("Error!!!");
			}

		   	this.pushStatusChange();
		}
    	},

   	get messageFlagsFlagged()
   	{
   		return (this._messageFlagsFlagged);
    	},
    		
    	set messageFlagsFlagged(newMessageFlagsFlagged)
    	{
    		if (newMessageFlagsFlagged != this._messageFlagsFlagged)
    		{
	   		this._messageFlagsFlagged = newMessageFlagsFlagged;
	   		
	   		if (catalog !== undefined)
	   		{
	   			catalog.runAction(this.identifier, ((newMessageFlagsFlagged==0) ? "markMessageUnflagged" : "markMessageFlagged"));
		   	}

		   	this.pushStatusChange();
		}
    	},

   	get messageFlagsDeleted()
   	{
   		return (this._messageFlagsDeleted);
    	},
    		
    	set messageFlagsDeleted(newMessageFlagsDeleted)
    	{
    		if (newMessageFlagsDeleted != this._messageFlagsDeleted)
    		{
	   		this._messageFlagsDeleted = newMessageFlagsDeleted;
	   		
	   		if (catalog !== undefined)
	   		{
	   			catalog.runAction(this.identifier, ((newMessageFlagsDeleted==0) ? "markMessageUndeleted" : "markMessageDeleted"));
		   	}

		   	this.pushStatusChange();
		}
    	}
};

CatalogItem.prototype.pushStatusChange = function()
{
	if (catalog !== undefined)
	{
		var sectionObj, rowObj;
	
		catalog.updateCounts();
		
		if ((sectionObj = catalog.sourcelist.getSectionByID("news")) !== null)
		{
			if ((rowObj = sectionObj.getRowByID(this.identifier)) !== null)
			{
				rowObj.imageURL = ((this.messageFlagsRead == 1) ? "iconRead.png" : "iconUnread.png");
			}
		}
	}
}

CatalogItem.prototype._itemDesc;
CatalogItem.prototype._updateWithItemDesc = function(itemDesc)
{
	this.type = itemDesc['type'];
	this.category = itemDesc['category'];
	this.status = itemDesc['status'];
	this.identifier = itemDesc['identifier'];
	this.name = itemDesc['name'];
	this.version = itemDesc['version'];
	this.description = ((itemDesc['description'] != "__null__")  ? itemDesc['description'] : null);
	this.descriptionURL = ((itemDesc['descriptionURL'] != "__null__")  ? itemDesc['descriptionURL'] : null);
	this.imageURL = ((itemDesc['imageURL']=='') ? null : itemDesc['imageURL']);
	this.date = itemDesc['date'];
	this.size = itemDesc['size'];
	this.sizeLocalized = itemDesc['sizeLocalized'];
	this.localCopyDownloaded = itemDesc['localCopyDownloaded'];
	this.localCopyComplete = itemDesc['localClocalCopyCompleteopyDownloaded'];
	this.installed = itemDesc['installed'];
	this.inUse = itemDesc['inUse'];
	this.installItem = itemDesc['installItem'];

	this.installerFlags = itemDesc['installerFlags'];
	this.installerRequiresQuit = itemDesc['installerFlagsRequiresQuit'];
	this.installerRequiresRelaunch = itemDesc['installerFlagsRequiresRelaunch'];
	this.installerRequiresReboot = itemDesc['installerFlagsRequiresReboot'];
	this.installerRequiresAdminPriviledges = itemDesc['installerFlagsRequiresAdminPriviledges'];

	this.messageFlags = itemDesc['messageFlags'];
	this._messageFlagsRead = itemDesc['messageFlagsRead'];
	this._messageFlagsFlagged = itemDesc['messageFlagsFlagged'];
	this._messageFlagsDeleted = itemDesc['messageFlagsDeleted'];
	
	this._itemDesc = itemDesc;
}

CatalogItem.prototype.type;
CatalogItem.prototype.category;
CatalogItem.prototype.status;
CatalogItem.prototype.identifier;
CatalogItem.prototype.name;
CatalogItem.prototype.version;
CatalogItem.prototype.description;
CatalogItem.prototype.descriptionURL;
CatalogItem.prototype.imageURL;
CatalogItem.prototype.date;
CatalogItem.prototype.size;
CatalogItem.prototype.sizeLocalized;
CatalogItem.prototype.localCopyDownloaded;
CatalogItem.prototype.localCopyComplete;
CatalogItem.prototype.installed;
CatalogItem.prototype.inUse;
CatalogItem.prototype.installItem;

CatalogItem.prototype.installerFlags;
CatalogItem.prototype.installerRequiresQuit;
CatalogItem.prototype.installerRequiresRelaunch;
CatalogItem.prototype.installerRequiresReboot;
CatalogItem.prototype.installerRequiresAdminPriviledges;

CatalogItem.prototype.messageFlags;
CatalogItem.prototype._messageFlagsRead;
CatalogItem.prototype.messageFlagsFlagged;
CatalogItem.prototype.messageFlagsDeleted;

