/*
	ISUI - IOSPIRIT JavaScript UI
	Copyright (c) 2012-2014 IOSPIRIT GmbH. All rights reserved.
*/

/* Tools */
function ISGetElementsWithClassName(findClass, parent)
{
	var matches = [];

	if (parent !== undefined)
	{
		var elems = parent.getElementsByTagName('*');
		var i, elemCount;
		
		elemCount = elems.length;
		
		for (i=0; i < elemCount; i++)
		{
			if ((' ' + elems[i].className + ' ').indexOf(findClass) > -1)
			{
				matches.push(elems[i]);
			}
		}
	}
	
	return (matches);
}

function ISGetParentElementsWithClassName(findClass, descendant)
{
	var matches = [];
	var elem = descendant;
	
	while (elem !== null)
	{
		if ((' ' + elem.className + ' ').indexOf(findClass) > -1)
		{
			matches.push(elem);
		}
		
		elem = elem.parentNode;
	}
	
	return (matches);
}

/* SourceList
   ==========
   
   Expected div layout:
   	+ sourcelist
   	|
   	+-+ section
   	| +-- header - header of section
   	| +-+ rows   - rows container
   	|   +-- row  - row content
   	|
   	+-+ section
   	..
*/

function ISSourceList(sourceListObject)
{
	var myself = this;

	sourceListObject.setAttribute("ISSourceList", this);

	this._sections = [];

	this._container = sourceListObject;
	this._selectedSectionID = null;
	this._selectedRowID = null;
	this._muteNotifySelectionChanged = false;
	this._hideSectionHeaders = false;
	this.onViewClick = null;
	this.onSelectionChange = null;
	
	addEventListener("resize", function(event) { myself.layoutObject(false, false); });
}

ISSourceList.prototype = {
	get selectedSectionID()
	{
		return (this._selectedSectionID);
	},

	set selectedSectionID(sectionID)
	{
		if (this._selectedSectionID != sectionID)
		{
			this._selectedSectionID = sectionID;
			
			if (!this._muteNotifySelectionChanged)
			{
				this._muteNotifySelectionChanged=true;
				this.selectedRowID = this.getSectionByID(sectionID).selectedRowID;
				this._muteNotifySelectionChanged=false;
			}
			else
			{
				this.selectedRowID = this.getSectionByID(sectionID).selectedRowID;
			}

			this.notifySelectionChange(sectionID, this.selectedRowID);

			this.layoutObject(true, false);
		}
	},

	get selectedRowID()
	{
		return (this._selectedRowID);
	},

	set selectedRowID(rowID)
	{
		if (this._selectedRowID != rowID)
		{
			var sectionObj;
		
			this._selectedRowID = rowID;

			if ((sectionObj = this.getSectionByID(this.selectedSectionID)) !== null)
			{
				if (sectionObj.selectedRowID != rowID)
				{
					sectionObj.selectedRowID = rowID;
				}
			}
			
			this.notifySelectionChange(this.selectedSectionID, rowID);
		}
	},

	get hideSectionHeaders()
	{
		return (this._hideSectionHeaders);
	},

	set hideSectionHeaders(hideSectionHeaders)
	{
		if (this._hideSectionHeaders != hideSectionHeaders)
		{
			this._hideSectionHeaders = hideSectionHeaders;
			
			if (this._selectedSectionID != null)
			{
				this.render();
			}
		}
	},
};

ISSourceList.prototype._muteNotifySelectionChanged;
ISSourceList.prototype._selectedSectionID;
ISSourceList.prototype._selectedRowID;
ISSourceList.prototype._hideSectionHeaders;

ISSourceList.prototype.clear = function()
{
	this._sections = [];
	this._selectedSectionID = null;
	this._selectedRowID = null;
}

ISSourceList.prototype.setSelection = function(sectionID, rowID, notify)
{
	this._muteNotifySelectionChanged = true;
	this.selectedSectionID = sectionID;
	this.selectedRowID = rowID;
	this._muteNotifySelectionChanged = false;
	
	if (notify)
	{
		this.notifySelectionChange(sectionID, rowID);
	}
}

ISSourceList.prototype._container;
ISSourceList.prototype.layoutObject = function(animate, installHandlers)
{
	var sectionContainers, i, sectionCnt, sectionHeaderHeight = 0, myself=this, passedSelectedSection = false;

	sectionContainers = ISGetElementsWithClassName("section", this._container);
	sectionCnt = sectionContainers.length;
	
	for (i=0; i<sectionCnt; i++)
	{
		var sectionContainer = sectionContainers[i];
		var headerContainer = ISGetElementsWithClassName("header", sectionContainer)[0];
		var rowsContainer = ISGetElementsWithClassName("rows", sectionContainer)[0];
		var rowContainers = undefined;
		var sectionNewTop, sectionNewBottom;
		var sectionID = sectionContainer.getAttribute("sectionID");
		
		// Layout
		if (i==0)
		{
			if (headerContainer !== undefined)
			{
				sectionHeaderHeight = headerContainer.offsetHeight;
			}
			else
			{
				sectionHeaderHeight = 0;
			}
		}

		if (passedSelectedSection === false)
		{
			sectionNewTop = (sectionHeaderHeight * i) + "px";
		}
		else
		{
			sectionNewTop = (this._container.offsetHeight - ((sectionCnt-i) * sectionHeaderHeight)) + "px";
		}

		sectionNewBottom = (((sectionCnt-i-1) * sectionHeaderHeight) -1) + "px";
		
		if (rowsContainer !== undefined)
		{
			rowsContainer.style.top = sectionHeaderHeight;
			rowsContainer.style.height = (this._container.offsetHeight - (sectionCnt * sectionHeaderHeight) - 1) + "px";
		}
		
		if (animate === true)
		{
			sectionContainer.style.webkitTransitionDuration = "0.15s";
			sectionContainer.style.top = sectionNewTop;
			sectionContainer.style.bottom = sectionNewBottom;
		}
		else
		{
			sectionContainer.style.webkitTransitionDuration = "0.0s";
			sectionContainer.style.top = sectionNewTop;
			sectionContainer.style.bottom = sectionNewBottom;
		}

		sectionContainer.style.zIndex = i;
		
		// Set handlers
		if (installHandlers)
		{
			var sectionObj = null;
			
			if (sectionID !== undefined)
			{
				if ((sectionObj = this.getSectionByID(sectionID)) !== null)
				{
					sectionObj._sectionContainer = sectionContainer;
					sectionObj._headerContainer = headerContainer;
					sectionObj._rowsContainer = rowsContainer;
				}
			}
		
			// Clicks on section headers
			if (headerContainer !== undefined)
			{
				headerContainer.setAttribute("ISSectionIndex", i);
				headerContainer.onclick = function(event)
				{
					myself.handleClick(event, "section");
				};
			}
			
			// Switch selection
			if (rowsContainer !== undefined)
			{
				rowContainers = ISGetElementsWithClassName("row", rowsContainer);

				if (rowContainers !== undefined)
				{
					var rowIndex, rowCount = rowContainers.length;
					
					for (rowIndex=0; rowIndex<rowCount; rowIndex++)
					{
						var rowObj, rowContainer;
						
						rowContainer = rowContainers[rowIndex];
						
						rowContainer.onclick = function(event)
						{
							myself.handleClick(event, "row");
						}
						
						if (sectionObj !== undefined)
						{
							var rowID = rowContainer.getAttribute("rowID");
							
							rowObj = sectionObj.getRowByID(rowID);
						
							if (rowObj !== null)
							{
								rowObj._rowContainer = rowContainer;
							}
						}
					}
				}
			}
		}
		
		if (sectionID == this.selectedSectionID)
		{
			passedSelectedSection = true;
		}
	}
}

ISSourceList.prototype.handleClick = function(event, targetType)
{
	if (targetType == "section")
	{
		var sectionIndex = event.target.getAttribute("ISSectionIndex");
		var sectionID = (ISGetParentElementsWithClassName("section", event.target)[0]).getAttribute("sectionID");
			
		if (this.selectedSectionID != sectionID)
		{
			this.selectedSectionID = sectionID;
		}

		this.notifyClick(event, targetType, sectionID, null);
	}
	
	if (targetType == "row")
	{
		var rowID = (ISGetParentElementsWithClassName("row", event.target)[0]).getAttribute("rowID");
		var sectionID = (ISGetParentElementsWithClassName("section", event.target)[0]).getAttribute("sectionID");
		var sectionObj;
		
		if ((sectionObj = this.getSectionByID(sectionID)) !== null)
		{
			sectionObj.selectedRowID = rowID;
			this.selectedRowID = rowID;
		}
		
		this.notifyClick(event, targetType, sectionID, rowID);
	}
}

ISSourceList.prototype.notifyClick = function(event, targetType, sectionID, rowID)
{
	// alert("Click: " + sectionID +" - " + rowID);
	if (this.onViewClick !== null)
	{
		this.onViewClick(sectionID, rowID);
	}
}

ISSourceList.prototype.notifySelectionChange = function(sectionID, rowID)
{
	if (!this._muteNotifySelectionChanged)
	{
		if (this.onSelectionChange !== null)
		{
			this.onSelectionChange(sectionID, rowID);
		}
	}
}

ISSourceList.prototype.onSelectionChange; // f.ex. sourceList.onSelectionChange = function(sectionID, rowID) ..
ISSourceList.prototype.onViewClick; // f.ex. sourceList.onViewClick = function(event, targetType, sectionID, rowID) ..

ISSourceList.prototype._sections;
ISSourceList.prototype.addSection = function(sectionID, title)
{
	var newSection;
	
	newSection = new ISSourceListSection(sectionID);
	newSection.title = title;
	
	this._sections.push(newSection);
	
	return (newSection);
}

ISSourceList.prototype.getSectionByID = function(sectionID)
{
	var i;
	
	for (i=0;i<this._sections.length;i++)
	{
		if (this._sections[i].sectionID == sectionID)
		{
			return (this._sections[i]);
		}
	}
	
	return (null);
}

ISSourceList.prototype.render = function()
{
	var listCode = "", i, sectionCnt;
	
	sectionCnt = this._sections.length;
	
	if ((this.selectedSectionID === null) && (sectionCnt > 0))
	{
		this.selectedSectionID = this._sections[0].sectionID;
	}

	for (i=0; i<sectionCnt; i++)
	{
		var sectionObj = this._sections[i];
		var rowCnt;
	
		listCode += '<div class="section" sectionID="' + sectionObj.sectionID + '">';
		if (!this.hideSectionHeaders)
		{
			listCode += '<div class="header">';
			if (sectionObj.imageURL !== null)
			{
				listCode += '<div class="image" style="background-image: url(' + sectionObj.imageURL + ');"></div>';
			}
			if (sectionObj.badge !== null)
			{
				listCode += '<div class="badge">' + sectionObj.badge + '</div>';
			}
			else
			{
				listCode += '<div class="badge" style="display:none;"></div>';
			}
			listCode += sectionObj.title;
			listCode += '</div>';
		}

		listCode += '<div class="rows">';
		if ((rowCnt = sectionObj._rows.length) > 0)
		{
			var row;
			
			for (row=0;row<rowCnt;row++)
			{
				var rowObj = sectionObj._rows[row];
				
				listCode += '<div class="row" selected="' + ((sectionObj.selectedRowID == rowObj.rowID) ? 'yes' : 'no') + '" rowID="' + rowObj.rowID + '">';
				listCode += rowObj.generateRowHTML();
				listCode += '</div>'; // row
			}
		}
		listCode += '</div>'; // rows
		
		// detect "updates” caused by re-creating objects
		if (sectionObj.sectionID == this.selectedSectionID)
		{
			if (sectionObj.selectedRowID != this.selectedRowID)
			{
				this.selectedRowID = sectionObj.selectedRowID;
			}
		}
		
		listCode += '</div>'; // section
	}
	
	this._container.innerHTML = listCode;

	this.layoutObject(false, true);
}

function ISInitSourceLists()
{
	var sourceListContainers, i;

	sourceListContainers = ISGetElementsWithClassName("sourcelist", document);

	for (i=0; i<sourceListContainers.length; i++)
	{
		var sourceList;
		
		// Only automatically turn sourcelists into ISSourceLists who have autosourcelist set
		if (sourceListContainers[i].getAttribute("autosourcelist"))
		{
			sourceList = new ISSourceList(sourceListContainers[i]);
		}
	}
}

addEventListener("DOMContentLoaded", function(event) { ISInitSourceLists(); }, false); // DOMContentLoaded fires even before onLoad

/* ISSourceList: section and row objects */
// Section object
function ISSourceListSection(sectionID)
{
	this.sectionID = sectionID;
	this._rows = [];
	this.allowEmptySelection = false;
	this._selectedRowID = null;
	this._badge = null;
}

ISSourceListSection.prototype = {
   	get selectedRowID()
   	{
   		if ((this.allowEmptySelection==false) && ((this._selectedRowID === undefined) || (this._selectedRowID === null)))
   		{
   			if (this._rows.length > 0)
   			{
	   			this._selectedRowID = this._rows[0].rowID;
   			}
   		}
   	
    		return (this._selectedRowID);
    	},
    		
    	set selectedRowID(rowID)
    	{
		var i;
		
		for (i=0;i<this._rows.length;i++)
		{
			if (this._rows[i]._rowContainer !== null)
			{
				if (this._rows[i].rowID == rowID)
				{
					this._rows[i]._rowContainer.setAttribute("selected", "yes");
				}
				else
				{
					this._rows[i]._rowContainer.setAttribute("selected", "no");
				}
			}
		}
		
		this._selectedRowID = rowID;
    	},
	
	get badge()
	{
		return (this._badge);
	},
	
	set badge(newBadge)
	{
		if (newBadge != this._badge)
		{
			this._badge = newBadge;
			
			if ((this._headerContainer !== null) && (this._headerContainer !== undefined))
			{
				var badgeContainers = ISGetElementsWithClassName("badge", this._headerContainer);
				
				if (badgeContainers.length > 0)
				{
					var badgeContainer = badgeContainers[0];
					
					if (this._badge !== null)
					{
						badgeContainer.innerHTML = this._badge;
						badgeContainer.style.display = "block";
					}
					else
					{
						badgeContainer.innerHTML = "";
						badgeContainer.style.display = "none";
					}
				}
			}
		}
	}
};

ISSourceListSection.prototype._rows;
ISSourceListSection.prototype._sectionContainer;
ISSourceListSection.prototype._headerContainer;
ISSourceListSection.prototype._rowsContainer;
ISSourceListSection.prototype._badge;
ISSourceListSection.prototype.sectionID;
ISSourceListSection.prototype.getRowCount = function()
{
	return (this._rows.length);
}
ISSourceListSection.prototype.getRowByID = function(rowID)
{
	var i;
	
	for (i=0;i<this._rows.length;i++)
	{
		if (this._rows[i].rowID == rowID)
		{
			return (this._rows[i]);
		}
	}
	
	return (null);
}
ISSourceListSection.prototype.title;
ISSourceListSection.prototype.imageURL;
ISSourceListSection.prototype.allowEmptySelection;
ISSourceListSection.prototype._selectedRowID;

ISSourceListSection.prototype.addRow = function(rowID, rowTitle)
{
	var newRow;
	
	newRow = new ISSourceListRow(this, rowID);
	newRow.title = rowTitle;
	
	this._rows.push(newRow);
	
	return (newRow);
}

// Row objects
function ISSourceListRow(sectionObj, rowID)
{
	this.section = sectionObj;
	this.rowID = rowID;
	this._imageURL = null;
	this._title = null;
	this._rowContainer = null;
}

ISSourceListRow.prototype = {
   	get title()
   	{
   		return (this._title);
  	},
    		
    	set title(newTitle)
    	{
    		this._title = newTitle;
    		
    		this._updateRow();
    	},

   	get imageURL()
   	{
   		return (this._imageURL);
  	},
    		
    	set imageURL(newImageURL)
    	{
    		this._imageURL = newImageURL;

    		this._updateRow();
    	}
};

ISSourceListRow.prototype.generateRowHTML = function()
{
	var listCode = "";

	listCode += '<div class="text">';
	if (this.imageURL !== null)
	{
		listCode += '<div class="image" style="background-image: url(' + this.imageURL + ');"></div>';
	}
	if (this.title !== null)
	{
		listCode += this.title;
	}
	listCode += '</div>'; // text
	
	return (listCode);
}

ISSourceListRow.prototype._updateRow = function()
{
	if (this._rowContainer !== null)
	{
		this._rowContainer.innerHTML = this.generateRowHTML();
	}
}

ISSourceListRow.prototype.section;
ISSourceListRow.prototype.rowID;
ISSourceListRow.prototype._title;
ISSourceListRow.prototype._imageURL;
ISSourceListRow.prototype._rowContainer;
